using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Objects;
using Luski.GUI.MainScreen.UI.LuskiControls;
using Luski.net.Structures.Public;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;

namespace Luski.GUI.MainScreen.UI.PublicServers;

public class PublicChat : UserControl
{

    public FlowLayout MessageFlow;
    private Label title, desc;
    private TextBox tb;
    private SocketChannel? Channel;
    UserControl titlecon;
    private Rectangle? UserCon;
    
    public PublicChat()
    {
        
        base.Size = new((int)(980 * Globals.Settings.Scale), (int)(866 * Globals.Settings.Scale));
        BackgroundColor = new(50, 50, 50, 255);
        Anchor = ObjectAnchor.All;
        Controls.Add(MessageFlow = new()
        {
            Size = new(base.Size.X, (int)(761 * Globals.Settings.Scale)),
            Location = new(0, (int)(52 * Globals.Settings.Scale), 0),
            BackgroundColor = new(40,40,40,255),
            Anchor = ObjectAnchor.All,
            HScrollPixels = Globals.Settings.PerScrollPixels,
        });
        
        Controls.Add(titlecon = new UserControl()
        {
            Anchor = ObjectAnchor.Left | ObjectAnchor.Top | ObjectAnchor.Right,
            Size = new((int)(980 * Globals.Settings.Scale), (int)(48 * Globals.Settings.Scale)),
            BackgroundColor = BackgroundColor
        });
        if (LuskiExperiments.ServerExperiments.MemberList.IsEnabled())
        {
            UserCon =
                new(Globals.ms.TextureManager.GetTextureResource("person.png"))
                {
                    Size = new((int)(24 * Globals.Settings.Scale)),
                    Location = new((int)(944 * Globals.Settings.Scale),(int)(12 * Globals.Settings.Scale),0),
                    Anchor = ObjectAnchor.Right | ObjectAnchor.Top,
                    Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
                    BackgroundColor = Color4.Red
                };
            titlecon.Controls.Add(UserCon);
        }
        LuskiExperiments.ServerExperiments.MemberList.EventToggled += MemberListOnEventToggled;
        
        titlecon.ForceDistanceUpdate(this);
        titlecon.Controls.Add(title = new Label(Globals.DefaultFont)
        {
            //Location = new(
            // (int)((26 - (Globals.DefaultFont.PixelHeight / 2)) * Globals.Settings.Scale),
            // (int)((26 * Globals.Settings.Scale) - (Globals.DefaultFont.PixelHeight/2.0)))
        });
        titlecon.Controls.Add(desc = new Label(Globals.DefaultFont)
        {
            Color = new(161,161,161,255), 
            Location = new(title.Location.X + title.Size.X + 5, title.Location.Y, 0)
        });
        
        Controls.Add(tb = new()
        {
           // InsideColor = new(28, 28, 28, 255),
            //BorderColor = Color4.DarkCyan,
            Location = new((int)(10 * Globals.Settings.Scale), (int)(824 * Globals.Settings.Scale), 0),
            Size = new((int)(960 * Globals.Settings.Scale), (int)(34 * Globals.Settings.Scale)),
            Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left | ObjectAnchor.Right,
            HoverMouse = MouseCursor.IBeam,
            //Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            BackgroundColor = Color4.Red
        });
        tb.ForceDistanceUpdate(this);
        //tb.KeyPress += TbOnKeyPress;
        //Globals.Luski.MainServer.MessageReceived += LuskiOnMessageReceived;
    }

    private Task MemberListOnEventToggled(bool arg)
    {
        if (arg)
        {
            UserCon =
                new(Globals.ms.TextureManager.GetTextureResource("person.png"))
                {
                    Size = new((int)(24 * Globals.Settings.Scale)),
                    Location = new(base.Size.X - (int)(36 * Globals.Settings.Scale),(int)(12 * Globals.Settings.Scale),0),
                    Anchor = ObjectAnchor.Right | ObjectAnchor.Top,
                    Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
                    BackgroundColor = Color4.Red
                };
            titlecon.Controls.Add(UserCon);
        }
        else
        {
            titlecon.Controls.Remove(UserCon!);
            UserCon = null;
        }

        return Task.CompletedTask;
    }

    private SocketMessage? lastm;
    private long? lastUser;
    private ChatMessage? LastChatMessage;

    public async Task LoadChannel(SocketChannel channel)
    {
        Channel = channel;
        title.Text = channel.Name;
        desc.Text = channel.Description;
        desc.Location = new((int)(title.Location.X + title.Size.X + (5 * Globals.Settings.Scale)), desc.Location.Y, 0);
        if (Window is not null)
        {
            Window.Title = $"{channel.Name} | {channel.Server.Name} - Luski";
        }
        tb.WatermarkText = $"Message {channel.Name}";
    }

    public void ClearChat()
    {
        MessageFlow.Controls.Clear();
        lastm = null;
        lastUser = null;
        LastChatMessage = null;
    }

    public async Task AddMessages(IEnumerable<SocketMessage> messages, bool reverse = true)
    {
        MessageFlow.BlockDraw = true;
        if (reverse)
        {
            foreach (SocketMessage message in messages.Reverse())
            {
                await AddMessage(message);
            }
        }
        else
        {
            foreach (SocketMessage message in messages)
            {
                await AddMessage(message);
            }
        }
        MessageFlow.BlockDraw = false;
    }
    
    public async Task AddMessage(SocketMessage Message)
    {
        bool hasbeentenmin = false;
        if (lastm is not null)
            hasbeentenmin =
                Channel!.Epoch.AddMilliseconds(lastm.ID >> 22).ToLocalTime().AddMinutes(10) <
                Channel!.Epoch.AddMilliseconds(Message.ID >> 22).ToLocalTime();
        lastm = Message;
        if (lastUser is null || lastUser != Message.AuthorID || hasbeentenmin)
        {
            if (Window is null || !Globals.ms.InvokeRequired)
            {
                MessageFlow.Controls.Add(LastChatMessage = await ChatMessage.MakeChatMessage(this, Message));
            }
            else
            {
                LastChatMessage = await ChatMessage.MakeChatMessage(this, Message);
                Globals.ms.Invoke(() =>
                {
                    MessageFlow.Controls.Add(LastChatMessage);
                    Window.TryDraw();
                });
            }
        }
        else
        {
            if (Window is null || !Globals.ms.InvokeRequired)
            {
                await LastChatMessage!.AddMessage(Message);
            }
            else
            {
                Globals.ms.Invoke(() =>
                {
                    LastChatMessage!.AddMessage(Message);
                    Window!.TryDraw();
                });
            }
        }

        lastUser = Message.AuthorID;
    }
}