using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.Classes;
using Luski.GUI.MainScreen.UI.SettingsPanel;
using OpenTK.Mathematics;

namespace Luski.GUI.MainScreen.UI;

public class SettingsMenu : UserControl
{
    private string BehindName;
    public FlowLayout page;
    public CategoryButton? Selected;
    private FlowLayout fl;
    private Category? AppSettings;
    private FontInteraction f;
    public SettingsMenu()
    {
        LuskiExperiments.Settings.Theme.EventToggled += ThemeOnEventToggled;
        BehindName = Globals.ms.Title;
        Globals.ms.Title = "Settings - Luski";
        BackgroundColor = new(34, 34, 34, 255);
        base.Size = Globals.ms.Size;
        Anchor = ObjectAnchor.All;
        if (CategoryButton.seltec is null)
        {
            CategoryButton.seltec = Globals.ms.TextureManager.GetTextureResource("RoundedRectangle.png");
        }
        fl = new()
        {
            BackgroundColor = new(20, 20, 20, 255),
            Size = new((int)(307 * Globals.Settings.Scale), base.Size.Y),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom
        };
        f = Globals.DefaultFont.Clone();
        f.FontSize = FontSize.Bold;
        f.PixelHeight = (uint)(f.PixelHeight * 1.4f);


        if (LuskiExperiments.Settings.Theme.IsEnabled())
        {
            AppSettings = new("APP SETTINGS");
            CategoryButton cb2 = new("Appearance", this)
            {
                OnPageLoad = () =>
                {
                    page!.Controls.Add(new Label(f)
                    {
                        Text = " \nAppearance\n "
                    });
                }
            };
            AppSettings.AddButton(cb2);
            fl.Controls.Insert(0, AppSettings);
            fl.ScrollToBottom();
        }
        
        
        Category As = new("ADVANCED SETTINGS");
        CategoryButton cb = new("Experiments", this)
        {
            OnPageLoad = () =>
            {
                page!.Controls.Add(new Label(f)
                {
                    Text = " \nExperiments\n "
                });
                ExperimentGUI? g = null;
                foreach (ExperimentInfo exp in Globals.Experiments)
                {
                    g = new(exp);
                    page.Controls.Add(g);
                }

                if (g is not null)
                {
                    g.line.WindowLoaded += _ =>
                    {
                        page.ParentResize(new(Globals.ms.Size));
                        return Task.CompletedTask;
                    };
                }
            }
        };
        CategoryButton us = new("Updater Config", this)
        {
            OnPageLoad = () =>
            {
                page!.Controls.Add(new Label(f)
                {
                    Text = " \nUpdater Config\n "
                });
            }
        };
        As.AddButton(cb);
        As.AddButton(us);
        fl.Controls.Add(As);
        page = new()
        {
            BackgroundColor = this.BackgroundColor,
            Location = new(fl.Size.X + (int)(40 * Globals.Settings.Scale), 0, 0),
            Size = new(Globals.ms.Size.X - fl.Size.X - ((int)(80 * Globals.Settings.Scale)), Globals.ms.Size.Y),
            AllowHoverFromBehind = true,
            Anchor = ObjectAnchor.All,
            HScrollPixels = Globals.Settings.PerScrollPixels
        };
        Controls.Add(page);
        _ = cb.ToggleSelected();
        fl.ForceDistanceUpdate(this);
        
        Rectangle closebtn = new(Globals.ms.TextureManager.GetTextureResource("close.png"))
        {
            Location = new(Globals.ms.Size.X - (int)(40 * Globals.Settings.Scale), (int)(8 * Globals.Settings.Scale),0),
            Size = new((int)(32 * Globals.Settings.Scale)),
            Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            BackgroundColor = Color4.Gray,
            Anchor = ObjectAnchor.Top | ObjectAnchor.Right
        };
        closebtn.MouseEnter += _ =>
        {
            closebtn.BackgroundColor = Color4.White;
            return Task.CompletedTask;
        };
        closebtn.MouseLeave += _ =>
        {
            closebtn.BackgroundColor = Color4.Gray;
            return Task.CompletedTask;
        };
        closebtn.Clicked += ClosebtnOnClicked;
        closebtn.ForceDistanceUpdate(this);
        Controls.Add(closebtn);
        Controls.Add(fl);
    }

    private Task ThemeOnEventToggled(bool arg)
    {
        if (arg)
        {
            AppSettings = new("APP SETTINGS");
            CategoryButton cb = new("Appearance", this)
            {
                OnPageLoad = () =>
                {
                    page.Controls.Add(new Label(f)
                    {
                        Text = " \nAppearance\n "
                    });
                }
            };
            AppSettings.AddButton(cb);
            fl.Controls.Insert(0, AppSettings);
            fl.ScrollToBottom();
        }
        else
        {
            fl.Controls.Remove(AppSettings!);
        }

        return Task.CompletedTask;
    }

    private Task ClosebtnOnClicked(IRenderObject arg)
    {
        Globals.ms.Controls.Remove(this);
        Globals.ms.Title = BehindName;
        Globals.ms.DrawFrame();
        return Task.CompletedTask;
    }
}